#!/usr/bin/env python
# -*- coding: utf-8 -*-

import asyncio
import logging
from concurrent.futures import ThreadPoolExecutor

logger = logging.getLogger(__name__)


class TaskExecuter:
    def __init__(self, threads):
        self.pool = ThreadPoolExecutor(threads)
        self.results = []
        self.tasks = []

    async def execute_task(self, func, **kwargs):
        future = self.pool.submit(func, **kwargs)
        awaitable = asyncio.wrap_future(future)
        return await awaitable

    def add_task(self, func):
        self.tasks.append(func)

    async def app(self):
        result = await asyncio.gather(*self.tasks)
        self.results.append(result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import secrets

import requests

from XOTweet.core.exceptions import WrongStatusCodeException
from XOTweet.core.settings import AUTHORIZATION_TOKEN, GUEST_TOKEN_URL
from XOTweet.core.user_agents import user_agents

logger = logging.getLogger(__name__)


def get_random_user_agent() -> str:
    return secrets.choice(user_agents)


def get_guest_token():
    try:
        resp = requests.post(GUEST_TOKEN_URL, headers={
            "authorization": "bearer {}".format(AUTHORIZATION_TOKEN)})
        if resp.status_code == 200:
            return resp.json().get("guest_token", None)
        raise WrongStatusCodeException
    except Exception as e:
        logger.error(e)
        raise e


def get_headers(new_guest_token: bool = True, guest_token: str = "") -> dict:
    if new_guest_token:
        guest_token = get_guest_token()
    headers = {
        "User-Agent": get_random_user_agent(),
        "x-guest-token": guest_token,
        "x-twitter-client-language": "en",
        "x-twitter-active-user": "yes",
        "authorization": "Bearer {}".format(AUTHORIZATION_TOKEN),
        "referer": "https://twitter.com/"
    }
    return headers


def form_query(query: str = "", exclude: str = None, from_date: str = None, till_date: str = None) -> str:
    if exclude is None:
        exclude = []
    for x in exclude:
        query += " exclude:{}".format(x)
    if from_date:
        query += " since:{}".format(from_date)
    if till_date:
        query += " until:{}".format(till_date)
    return query



# !/usr/bin/env python
# -*- coding: utf-8 -*-

from argparse import ArgumentParser, Namespace, ArgumentTypeError
from datetime import datetime
from typing import Tuple, List


def strtobool(v):
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise ArgumentTypeError('Boolean value expected.')


def strToList(v):
    items = v.split(",")
    if len(items) >= 1:
        return items
    else:
        raise ArgumentTypeError('Expected list')


def strToDateTime(v):
    return datetime.strptime(v, '%Y-%m-%d').date()


def parse_args() -> Tuple[Namespace, List[str]]:
    """ Parses the args from command line
    Returns:
        Tuple[Namespace, List[str]]: Returns known parsed
        arguments and a list of unknown args
    """
    parser = ArgumentParser()

    parser.add_argument(
        "Keyword",
        metavar='Keyword',
        type=str,
        help="Give keyword to fetch results")

    parser.add_argument(
        "-f",
        dest="Format",
        required=False,
        default="json",
        help="Output format")

    parser.add_argument(
        "-x",
        dest="Exclude",
        required=False,
        default=[],
        type=strToList,
        nargs='?',
        help="exclude re-tweets and replies")

    parser.add_argument(
        "-d",
        dest="From_Date",
        required=False,
        default=None,
        type=strToDateTime,
        nargs='?',
        help="date to show tweets from")

    parser.add_argument(
        "-t",
        dest="Threads",
        required=False,
        default=5,
        type=int,
        help="Number of threads")

    args, unknown = parser.parse_known_args()

    return args, unknown
